Package: xml2
Title: Parse XML
Version: 1.5.2
Authors@R: c(
    person("Hadley", "Wickham", role = "aut"),
    person("Jim", "Hester", role = "aut"),
    person("Jeroen", "Ooms", email = "jeroenooms@gmail.com", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd")),
    person("R Foundation", role = "ctb",
           comment = "Copy of R-project homepage cached as example")
  )
Description: Bindings to 'libxml2' for working with XML data using a simple, 
    consistent interface based on 'XPath' expressions. Also supports XML schema
    validation; for 'XSLT' transformations see the 'xslt' package.
License: MIT + file LICENSE
URL: https://xml2.r-lib.org, https://r-lib.r-universe.dev/xml2
BugReports: https://github.com/r-lib/xml2/issues
Depends: R (>= 3.6.0)
Imports: cli, methods, rlang (>= 1.1.0)
Suggests: covr, curl, httr, knitr, mockery, rmarkdown, testthat (>=
        3.2.0), xslt
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Encoding: UTF-8
RoxygenNote: 7.3.3
SystemRequirements: libxml2: libxml2-dev (deb), libxml2-devel (rpm)
Collate: 'S4.R' 'as_list.R' 'xml_parse.R' 'as_xml_document.R'
        'classes.R' 'format.R' 'import-standalone-obj-type.R'
        'import-standalone-purrr.R' 'import-standalone-types-check.R'
        'init.R' 'nodeset_apply.R' 'paths.R' 'utils.R' 'xml2-package.R'
        'xml_attr.R' 'xml_children.R' 'xml_document.R' 'xml_find.R'
        'xml_missing.R' 'xml_modify.R' 'xml_name.R' 'xml_namespaces.R'
        'xml_node.R' 'xml_nodeset.R' 'xml_path.R' 'xml_schema.R'
        'xml_serialize.R' 'xml_structure.R' 'xml_text.R' 'xml_type.R'
        'xml_url.R' 'xml_write.R' 'zzz.R'
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-01-17 13:03:05 UTC; jeroen
Author: Hadley Wickham [aut],
  Jim Hester [aut],
  Jeroen Ooms [aut, cre],
  Posit Software, PBC [cph, fnd],
  R Foundation [ctb] (Copy of R-project homepage cached as example)
Maintainer: Jeroen Ooms <jeroenooms@gmail.com>
Repository: CRAN
Date/Publication: 2026-01-17 16:10:02 UTC
