# print method is correct

    Code
      print(body)
    Output
      {html_node}
      <body id="styleguide-v2" class="fixed">
       [1] <script>\n    if (typeof uet == 'function') {\n      uet("bb");\n    }\n ...
       [2] <script>\n    if ('csm' in window) {\n      csm.measure('csm_body_delive ...
       [3] <div id="wrapper">\n            <div id="root" class="redesign">\n<scrip ...
       [4] <script type="text/javascript" src="http://ia.media-imdb.com/images/G/01 ...
       [5] <script type="text/imdblogin-js" id="login">\njQuery(document).ready(fun ...
       [6] <script type="text/javascript">\n                jQuery(\n               ...
       [7] <iframe id="sis_pixel_sitewide" width="1" height="1" frameborder="0" mar ...
       [8] <script>\n    setTimeout(function(){\n        try{\n            //sis3.0 ...
       [9] <script type="text/javascript" src="http://ia.media-imdb.com/images/G/01 ...
      [10] <script type="text/javascript">\nif(window.COMSCORE){\nCOMSCORE.beacon({ ...
      [11] <noscript>\n<img src="http://b.scorecardresearch.com/p?c1=2&amp;c2=60349 ...
      [12] <script>\n    doWithAds(function(){\n        (new Image()).src = "http:/ ...
      [13] <script>\n(function(){\n    var readyTimeout = setInterval(function(){\n ...
      [14] <div id="servertime" time="235"></div>
      [15] <script>\n    if (typeof uet == 'function') {\n      uet("be");\n    }\n ...

