% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{xml_missing}
\alias{xml_missing}
\title{Construct an missing xml object}
\usage{
xml_missing()
}
\description{
Construct an missing xml object
}
\keyword{internal}
